#!/usr/bin/env python
# -*- Encoding: latin-1 -*-
"""
Tests for the fraction class
 (the mk version, not the mh's one)
written 2004 by Marek Kubica
Licensed under GPL
"""

import unittest
import fractions as fract
__version__ = fract.__version__

class TestRepresentation(unittest.TestCase):
    """Test for checking if the fractions are properly inputed and outputted"""
    
    def testInputConstructor(self):
        """Test that the number constructor works"""
        whole, upper, lower = 1, 1, 2
        f = fract.Fraction(whole=whole, upper=upper, lower=lower)
        if f.getWhole() != whole:
            self.fail('setting whole failed')
        if f.getUpper() != upper:
            self.fail('setting upper failed')
        if f.getLower() != lower:
            self.fail('setting lower failed')
    
    def testInputConstructorString(self):
        """Test that the string constructor works"""
        # alternate constructor - uses directly strings
        w, u, l = 1, 1, 2
        #raise NotImplementedError
        string_input = str(w) + ' ' + str(u) + '/' + str(l)
        f = fract.Fraction(string_input)
        if f.getWhole() != w:
            self.fail('setting whole failed')
        elif f.getUpper() != u:
            self.fail('setting upper failed')
        elif f.getLower() != l:
            self.fail('setting lower failed')
    
    def testInputConstructorEmpty(self):
        """Test that the default values are set on construction"""
	# these are the default values (from the constructor of Fraction)
	# test also the use of reduce
        default = {'whole' : 0, 'upper' : 0, 'lower' : 1}
        f = fract.Fraction()
        if f.getWhole() != default['whole']:
            self.fail('setting whole failed')
        if f.getUpper() != default['upper']:
            self.fail('setting upper failed')
        if f.getLower() != default['lower']:
            self.fail('setting lower failed')
    
    def testInputString(self):
        """Test the input of string fractions"""
        w, u, l = 1, 1, 2
        string_input = str(w) + ' ' + str(u) + '/' + str(l)
        f = fract.Fraction()
        f.input(string_input)
        
        w_new, u_new, l_new = f.getWhole(), f.getUpper(), f.getLower()
        self.assertEqual(w, w_new, 'Whole does not compare equal')
        self.assertEqual(u, u_new, 'Upper does not compare equal')
        self.assertEqual(l, l_new, 'Lower does not compare equal')
    
    def testInputStringBad(self):
        """Test that input of bad string fractions gives errors"""
        w, u, l = 'a', 1, 2
        
        string_input = str(w) + ' ' + str(u) + '/' + str(l)
        f = fract.Fraction()
        
        # run f.input(string_input) and look for ValueError
        self.assertRaises(ValueError, f.input, string_input)
    
    def testInputStringLowerZero(self):
        """Test that it is possible to set negative fractions using strings"""
        f = fract.Fraction()
        w, u, l = -1, 1, 2
        string_input = str(w) + ' ' + str(u) + '/' + str(l)
        f.input(string_input)
        string_output = f.getValue()
        
        self.assertEqual(string_input, string_output, 'String streams not equal')
        #self.fail('test not ready')

    def testInputStringWholeNumber(self):
	"""Test setting a simple whole number using strings"""
	val_norm = '10'
	f = fract.Fraction(val_norm)
	val_prog = f.getValue()
	self.assertEqual(val_prog, val_norm, 'The value returned is bad')
	
    def testInputWholeNumber(self):
	"""Test setting a simple whole number"""
	val_norm = 10
	f = fract.Fraction(val_norm)
	val_prog = int(f.getValue())
	self.assertEqual(val_prog, val_norm, 'The value returned is bad')
	#raise NotImplementedError
    
    def testOutputString(self):
        """Test if the output string is correct"""
        # should this mean inputstring == outputstring or can the factors be reduced?

	input_string = '3 3/4'
	f = fract.Fraction(input_string)
	output_string = f.getValue()
	self.assertEqual(input_string, output_string, 'The returned string representation is bad')
    
    def testOutputPrettyprint(self):
	"""Test if the fraction prettyprinting is working"""
	inpt = '3 11/20'
	f = fract.Fraction(inpt)
	# use the internal __getPrettyprint function
	otpt = f._Fraction__getPrettyprint()
	# the prettyprinted string should be:
	pprint = '  11\n3 --\n  20'
	
	# look if the strings are the same
	self.assertEqual(pprint, otpt, 'Prettyprint is bad')
    
    def testSetWhole(self):
        """Test that setWhole() sets right values"""
        w, u, l = 1, 1, 2
        f = fract.Fraction(whole=w, upper=u, lower=l)
        w_new = f.getWhole()
        self.assertEqual(w, w_new)
    
    def testSetWholeBad(self):
	"""Test setting a bad 'whole' value"""
	w, u, l = 'trash', 1, 2
	#f = fract.Fraction(whole=w, upper=u, lower=l)
	self.assertRaises(ValueError, fract.Fraction, whole=w, upper=u, lower=l)
    
    def testSetUpper(self):
        """Test that setUpper() sets right values"""
        w, u, l = 1, 1, 2
        f = fract.Fraction(whole=w, upper=u, lower=l)
        u_new = f.getUpper()
        self.assertEqual(u, u_new)
    
    def testSetUpperBad(self):
        """Test that it is not possible to set bad values for upper"""
        w, u, l = 1, 'a', 2
        # ^^ upper is a string, do you see it?
        
        # try if we get a ValueError when we try to initialize with bad values (indriectly calling setUpper())
        self.assertRaises(ValueError, fract.Fraction, whole=w, upper=u, lower=l)
    
    def testSetLower(self):
        """Test that setLower() sets right values"""
        w, u, l = 1, 1, 2
        f = fract.Fraction(whole=w, upper=u, lower=l)
        l_new = f.getLower()
        self.assertEqual(l, l_new)
        #raise NotImplementedError
    
    def testSetLowerBad(self):
        """Test that it is not possible to set bad values for lower"""
        w, u, l = 1, 1, 'a'
        # ^^ lower is a string, do you see it?
        
        # try if we get a ValueError when we try to initialize with bad values (indriectly calling setLower())
        self.assertRaises(ValueError, fract.Fraction, whole=w, upper=u, lower=l)
    
    def testSetLowerBadZero(self):
        """Test that is is not possible to set zero values for lower"""
        w, u, l = 1, 1, 0
        # try to create a fraction with a lower == 0 value
        self.assertRaises(ValueError, fract.Fraction, whole=w, upper=u, lower=l)
        # ^^ Indriectly calling getLower() by __init__()
        
    def testNewRepresentation(self):
	"""Test the representation of the fraction"""
	fr = '3 3/4'
	f = fract.Fraction(fr)
	val_gv = f.getValue()
	val_repr = repr(f)
	self.assertEqual(val_gv, val_repr)

    def testZeroWholeGetValue(self):
	w, u, l = 0, 4, 5
	f = fract.Fraction(whole=w, upper=u, lower=l)
	w_new = f.getWhole()
	self.assertEqual(w, w_new, 'Whole is non-zero!')

    def testZeroWholeStringGetValue(self):
	fract_fract = '4/5'
	w = 0
	f = fract.Fraction(fract_fract)
	w_new = f.getWhole()
	self.assertEqual(w, w_new, 'Whole is non-zero!')
    
class TestFunction(unittest.TestCase):
    """Tests for functions to run on Fraction instances"""
    def testFloat(self):
        """Test that the fractions are flattened to proper float values"""
        w, u, l = 1, 1, 2
        fract_float = 1.5
        f = fract.Fraction(whole=w, upper=u, lower=l)
	
	try:
	    fract_float_new = float(f)
	except TypeError, e:
	    self.fail('Problems with __float__ hook: ' + str(e))
        self.assertEqual(fract_float, fract_float_new, 'Errors while converting to float')

    def testInt(self):
	"""Test that the fractions are flattened to proper int values"""
	fract_fract = '1 1/4'
	fract_int = 1
	f = fract.Fraction(fract_fract)
	try:
	    fract_int_new = int(f)
	except TypeError, e:
	    self.fail('Problems with __int__ hook: ' + str(e))
	self.assertEqual(fract_int, fract_int_new, 'Errors while converting to int')

    def testStr(self):
	"""Test the string version of the fraction"""
	fract_str = '1 1/4'
	f = fract.Fraction(fract_str)
	try:
	    fract_str_new = str(f)
	except TypeError, e:
	    self.fail('Problems with __repr__ hook: ' + str(e))
	self.assertEqual(fract_str, fract_str_new, 'Errors while converting to string')

    def testInternalUnreal(self):
	"""Test the internal function of making unreal fractions"""
	fract_fract = '3 3/4'
	fract_unreal = '15/4'
	f = fract.Fraction(fract_fract)
	f._Fraction__unreal()
	fract_unreal_new = f.getValue()
	self.assertEqual(fract_unreal, fract_unreal_new, 'Convertion into unreal fractions faulty')

    def testInternalMixed(self):
	fract_fract = '15/4'
	fract_mixed = '3 3/4'
	f = fract.Fraction(fract_fract)
	f._Fraction__mixed()
	fract_mixed_new = f.getValue()
	print fract_mixed_new
	self.assertEqual(fract_mixed, fract_mixed_new, 'Convertion into mixed numbers faulty')
	
    # test for using reduce function


class TestOperation(unittest.TestCase):
    # test adding, dividing and so on (overloaded types)
    pass

if __name__ == '__main__':
    unittest.main()